%
%
%  Super Resolution Localization Data Function for Imaris 7.5.0
%
%  Copyright Bitplane AG 2012
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Spots Functions">
%        <Item name="Super Resolution Localization Data To Spots" icon="Matlab">
%          <Command>MatlabXT::XTSuperResolutionLocalizationDataToSpots(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpSpots">
%          <Item name="Super Resolution Localization Data To Spots" icon="Matlab">
%            <Command>MatlabXT::XTSuperResolutionLocalizationDataToSpots(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
% 
%
%  Description:
%   
%   Loads Super Resolution Localization Data from a QuickPalm Particle Table File and creates Imaris Spots components.
% 
%

function XTSuperResolutionLocalizationDataToSpots(aImarisApplicationID)

%Connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end

%Read in File
[vFilename, vPathname] = uigetfile( ...
{  '*.*',  'All Files (*.*)'}, ...
   'Open File Dialog', ...
   'MultiSelect', 'on');

if vFilename == 0
  return
end

vSuperResolutionSpotFile = [vPathname, vFilename];

%Ask for Parameters

nl=sprintf('\n'); % be os independent

vFirstPrompt=['The spot radii will be estimated from the intensities in the file and the two following values: ' nl nl ...
              'Enter the full-width-half-maximum of the microscope point spread function (um):']; 
		 
vPrompt = {vFirstPrompt, ...
           'Enter the gain factor of the camera to convert particle intensities into photons:'};
vName = 'Parameters';
vNumlines = 1;
vDefaultAnswer = {'0.1', '1'};
vOptions.Resize='on';

vAnswer = inputdlg(vPrompt, vName, vNumlines, vDefaultAnswer, vOptions);
if isempty(vAnswer)
  return
end

vS  = str2double(vAnswer{1});
vG  = str2double(vAnswer{2});

try 
    file = imread(vSuperResolutionSpotFile); 
catch
    errordlg('The file or the file path are not valid.', 'File Open Error');
    return;
end
 
if (size(file, 1) < 1)
    msgbox('There are no particles in the file');
    return;
end

%Extract Spots from File
vProgressBar = waitbar(0.2,'Loading file ...');

vIntensities        = file(:,1);
% vpX                 = file(:,2);
% vpY                 = file(:,3);
vSpotsXYZ           = 1000*file(:,4:6);
% vLeftStdDev         = file(:,7);
% vRightStdDev        = file(:,8);
% vUpStdDev           = file(:,9);
% vDownStdDev         = file(:,10);
% vXSymmetry          = file(:,11);
% vYSymmetry          = file(:,12);
% vWidthMinusHeight   = file(:,13);
% vFrameNumber        = file(:,14)/1000000;
vRadii              = 1000*(vS)./sqrt((1/vG)*vIntensities);
vTimePoints         = zeros(1, numel(vIntensities));
clear file;

if any(isnan(vSpotsXYZ(:))) || any(isnan(vIntensities))
    warndlg('There are NaNs in the data');
end

%Create Components
vSpotsDataContainer       = vImarisApplication.GetFactory.CreateDataContainer;

vNofSpots = size(vSpotsXYZ,1);
vBucketSize = 2e6;
vNofBuckets = floor(vNofSpots / vBucketSize) + 1;

for vIteration = 1: vNofBuckets
  vProgressText = sprintf('Creating Spot Component %i of %i ...', vIteration, vNofBuckets);
  waitbar(0.4 + 0.4 / double(vNofBuckets) * double(vIteration), vProgressBar, vProgressText);
  vStart = (vIteration - 1)* vBucketSize + 1;
  vEnd = vIteration * vBucketSize;
  if vEnd > vNofSpots
    vEnd = vNofSpots;
  end
  vSpotsXYZPart = vSpotsXYZ(vStart:vEnd,:);
  vTimePointsPart = vTimePoints(vStart:vEnd);
  vRadiiPart = vRadii(vStart:vEnd);
  vSuperResolutionSpot = vImarisApplication.GetFactory.CreateSpots;
  vSuperResolutionSpot.Set(vSpotsXYZPart, vTimePointsPart, vRadiiPart);
  vSuperResolutionSpot.SetColorRGBA(255);
  vTitle = sprintf('Super Resolution Spots %i', vIteration);
  vSuperResolutionSpot.SetName(vTitle);
  vSpotsDataContainer.AddChild(vSuperResolutionSpot, -1);
end

vSpotsDataContainer.SetName(sprintf('Super Resolution Spots from %s', vFilename));

%Add Components to Surpass Scene
waitbar(0.9, vProgressBar, 'Adding to Scene');

%Add Components to Surpass Scene
vScene = vImarisApplication.GetSurpassScene;

if isempty(vScene)
  %Create Components
  vDataContainer  = vImarisApplication.GetFactory.CreateDataContainer;
  vLightSource    = vImarisApplication.GetFactory.CreateLightSource;
  vFrame          = vImarisApplication.GetFactory.CreateFrame;
  vVolume         = vImarisApplication.GetFactory.CreateVolume;
  vVolume.SetMode(Imaris.tMode.eModeMIP);
  vImarisApplication.SetSurpassScene(vDataContainer);
  vImarisApplication.GetSurpassScene.AddChild(vLightSource, -1);
  vImarisApplication.GetSurpassScene.AddChild(vFrame, -1);
  vImarisApplication.GetSurpassScene.AddChild(vVolume, -1);
  vScene = vImarisApplication.GetSurpassScene;
end

vScene.AddChild(vSpotsDataContainer, -1);

vImarisApplication.GetSurpassCamera.Fit();

close(vProgressBar);
